'use strict';

const Plugin = require('perf.plugin');
const elasticsearch = require('elasticsearch');
const util = require('util');
const get = require('lodash.get');
const shortid = require('shortid');
const moment = require('moment');

/**
 * {
 *   "es": {
 *     "host": "localhost:9200",
 *     "log": "trace"
 *   },
 *   "type": 'page_success',
 *   "rule": {
 *     "key": 'query.d.success',
 *     "value": "false"
 *   }
 * }
 * 
 */
module.exports = class childPlugin extends Plugin {
  constructor(config = {}) {
    super();
    this.config = config;
    this.esConfig = Object.assign({
      host: 'localhost:9200',
      requestTimeout: 200000
    }, config.es);
    this.filterType = config.type || 'page_success';
    this.rule = config.rule || {};
    this.connect();
  }

  connect () {
    this.client = new elasticsearch.Client(this.esConfig);
    console.info(`Elasticsearch: Adding connection to ${this.esConfig.host}`);
  }

  insert (msg, index, time) {
    let body = [
      { index: { _index: index, _type: 'log' } },
      { 'log': msg, date: time }
    ];

    this.client.bulk({ body })
    .catch(err => {
      console.log('es bulk error:', err);
    });
  }
  
  //所有业务函数逻辑都写在这里
  filter(data) {
    let type = util.isString(data.query.type) ? data.query.type : '';
    let { key, value } = this.rule;
    key = get(data, key);
    let appName = data.query.appName;
    let isvalidRule = (key === value) ? true : false;
    let isvalidType = (type === this.filterType) ? true : false;
    let now = data.createdAt ? moment(data.createdAt) : moment();
    let time = now.format();

    if (key && value && isvalidRule && isvalidType) {
      let index = `${appName}-${type}`;
      this.insert(data, index, time);
    }
    return data;
  }

  handle(_, data) {
    let result = this.filter(data);
    
    return result ? super.handle(result) : null;
  }
}
